/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code shows how to create a postscript file
representing the contents of a drawing area.
***************************************************************/

/* ps_demo.c */

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#define OFFSET 612

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

GC gc;
Widget toplevel;
Widget drawing_area;
Widget button;
Widget form;

void setup_gc()
/* set up the graphics context. */
{
   int foreground,background;
   XGCValues vals;
   Arg al[10];
   int ac;

   /* get the current fg and bg colors. */
   ac=0;
   XtSetArg(al[ac],XmNforeground,&foreground); ac++;
   XtSetArg(al[ac],XmNbackground,&background); ac++;
   XtGetValues(drawing_area,al,ac);

   /* create the gc. */
   vals.foreground = foreground;
   vals.background = background;
   gc=XtGetGC(drawing_area,GCForeground | GCBackground,&vals);
}

void PSInit()
/* start the postscript file. */
{
   printf("%%!PS-Adobe-2.0\n\n");
   printf("DrawLine {\n");
   printf("   gsave 1 setlinewidth 0 setgray moveto lineto stroke grestore\n");
   printf(" } bind def\n\n");
   printf("/DrawText {\n");
   printf("   gsave 1 setlinewidth 0 setgray moveto\n"); 
   printf("   /Courier findfont 12 scalefont setfont show grestore\n");
   printf(" } bind def\n\n");
   printf("/DrawArc {\n");
   printf("   gsave translate 1 setlinewidth 0 setgray\n");
   printf("   scale arc stroke grestore\n");
   printf(" } bind def\n\n");
   printf("612 0 translate\n");
   printf("90 rotate\n\n");
}

void PSDrawLine(x1,y1,x2,y2)
   int x1;
   int y1;
   int x2;
   int y2;
/* draw a line in postscript */
{
   printf("%d %d %d %d DrawLine\n",x1,OFFSET-y1,x2,OFFSET-y2);
}

void PSDrawRectangle(x,y,width,height)
   int x;
   int y;
   int width;
   int height;
/* draw a rectangle in postscript */
{
   printf("%d %d %d %d DrawLine\n",x,OFFSET-y,x+width,OFFSET-y);
   printf("%d %d %d %d DrawLine\n",x+width,OFFSET-y,x+width,
       OFFSET-y-height);
   printf("%d %d %d %d DrawLine\n",x+width,OFFSET-y-height,x,OFFSET-y-height);
   printf("%d %d %d %d DrawLine\n",x,OFFSET-y-height,x,OFFSET-y);
}

void PSDrawArc(x,y,width,height,a1,a2)
  int x;
  int y;
  int width;
  int height;
  int a1;
  int a2;
/* draw an arc in postscript */
{
   int radius, xrad, yrad, xscale, yscale;

   xrad=(int)width / 2;
   yrad=(int)height / 2;
   radius=(xrad < yrad) ? xrad : yrad;

   xscale=(int)(width > height) ? width/height : 1;
   yscale=(int)(height > width) ? height/width : 1;

   printf("0 0 %d %d %d %d %d %d %d DrawArc\n",
   radius,(int)a1/64,(int)a2/64,xscale,yscale,x+radius,OFFSET-y-radius);    
}

void PSDrawString(x,y,s)
   int x;
   int y;
   char *s;
/* draw an string in postscript */
{
   printf("(%s) %d %d DrawText\n",s,x,OFFSET-y);
}

void PSTerminate()
/* terminate the postscript file */
{
   printf("\nshowpage\n");
   printf("%%%%Trailer\n");
}

void buttonCB(w,client_data,call_data)
   Widget w;
   caddr_t client_data; 
   caddr_t call_data;
/* called when the "Create PostScript" button is pushed. Note
   the similarity between this function and exposeCB */
{
   PSInit();
   PSDrawLine(20,150,70,50);
   PSDrawLine(70,50,120,150);
   PSDrawLine(120,150,20,150);
   PSDrawRectangle(140,50,100,100);
   PSDrawArc(260,50,100,100,0,23040);
   PSDrawString(20,170,"Triangle");
   PSDrawString(140,170,"Rectangle");
   PSDrawString(260,170,"Circle");
   PSTerminate();
}

void exposeCB(w,client_data,call_data)
   Widget w;
   caddr_t client_data; 
   caddr_t call_data;
/* called whenever drawing area is exposed. */
{
   XDrawLine(XtDisplay(drawing_area),XtWindow(drawing_area), 
             gc, 20,150,70,50);
   XDrawLine(XtDisplay(drawing_area),XtWindow(drawing_area), 
             gc, 70,50,120,150);
   XDrawLine(XtDisplay(drawing_area),XtWindow(drawing_area), 
             gc, 120,150,20,150);
   XDrawRectangle(XtDisplay(drawing_area),XtWindow(drawing_area), 
                  gc, 140,50,100,100);
   XDrawArc(XtDisplay(drawing_area),XtWindow(drawing_area), 
            gc, 260,50,100,100,0,23040);
   XDrawString(XtDisplay(drawing_area),XtWindow(drawing_area), 
               gc, 20,170,"Triangle",8);
   XDrawString(XtDisplay(drawing_area),XtWindow(drawing_area), 
               gc, 140,170,"Rectangle",9);
   XDrawString(XtDisplay(drawing_area),XtWindow(drawing_area), 
               gc, 260,170,"Circle",6);
}

void main(argc,argv)
   int argc;
   char *argv[];
{
   Arg al[10];
   int ac;

   /* create the toplevel shell */
   toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,
       NULL,NULL,0);

   /* set window size. */
   ac=0;
   XtSetArg(al[ac],XmNheight,220); ac++;
   XtSetArg(al[ac],XmNwidth,380); ac++;
   XtSetValues(toplevel,al,ac);

   /* create a form to hold widgets */
   ac=0;
   form=XmCreateForm(toplevel,"form",al,ac);
   XtManageChild(form);    

   /* create a push button */
   ac=0;
   XtSetArg(al[ac],XmNlabelString,
   XmStringCreate("Push to generate PostScript",char_set)); ac++;
   XtSetArg(al[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
   XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
   XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
   button=XmCreatePushButton(form,"button",al,ac);
   XtManageChild(button);
   XtAddCallback(button,XmNactivateCallback,buttonCB,NULL);

   /* create a drawing area widget. */
   ac=0;
   XtSetArg(al[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
   XtSetArg(al[ac], XmNtopWidget, button); ac++;
   XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
   XtSetArg(al[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
   XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
   drawing_area=XmCreateDrawingArea(form,"drawing_area", al, ac);
   XtAddCallback(drawing_area,XmNexposeCallback,exposeCB,NULL);
   XtManageChild(drawing_area);

   setup_gc();

   XtRealizeWidget(toplevel);
   XtAppMainLoop(context);
}
